const moment = require('moment-timezone');
const ExpedientesAgendaUsuarios = require('../models/ExpedientesAgendaUsuarios');
const DespachoModel = require('../models/Despachos');
const { EventoHTML } = require('../Mail/EventoHTML.js');
const { sendMail } = require('../config/mail.js');
const RenovarHTMl = require('../Mail/RenovacionHTML');
const DESPACHO_APP = process.env.DESPACHO_APP;
const ConfiguracionesModel = require('../models/Configuraciones.js');
const UsuarioModel = require('../models/Usuarios.js');
const { getVentasXFechas } = require('./Cortes.Distribuidores.Controller.js');
const CorteModel = require('../models/CortesDistribuidores.js');
const VentaModel = require('../models/Ventas.js');

const AvisosAgenda = async (req, res) => {
  try {
    const fechaActualMexico = moment();

    const fechaUnMinutoDespues = moment().add(1, 'minute');

    const resultados = await ExpedientesAgendaUsuarios.find({
      estatus: 'Pendiente',
      fechaRecordatorio: {
        $gte: fechaActualMexico,
        $lt: fechaUnMinutoDespues
      }
    }).populate('usuario', 'email').populate('agenda');

    for (const expediente of resultados) {
      const { usuario, agenda } = expediente;
      const { email } = usuario;
      const { title, descripcion, fecha } = agenda;

      // Crear el HTML del evento
      const html = EventoHTML({
        title,
        description: descripcion,
        fecha: moment(fecha).format('DD/MM/YYYY'),
        hora: moment(fecha).format('HH:mm'),
        url: `${DESPACHO_APP}/agenda`
      });

      // Enviar el correo electrónico
      sendMail(html, 'Recordatorio de evento', email);

      // Actualizar el estatus del expediente
      // await ExpedientesAgendaUsuarios.findByIdAndUpdate(expediente._id, {}, { estatus: 'Enviado' });
    }

    // Convertir 'ahora' y 'fechaLimite' a String para una visualización correcta en la respuesta
    res.status(200).json({
      data: resultados,
      ahora: fechaActualMexico.format(),
      fechaLimite: fechaUnMinutoDespues.format()
    });
  } catch (error) {
    console.error('Error al realizar la búsqueda:', error);
    res.status(500).json({ message: 'Error al realizar la búsqueda' });
  }
};

const AvisoSuscripcion = async (req, res) => {
  try {
    const tomorrow = moment().add(1, 'day').startOf('day').toDate();

    const despachos = await DespachoModel.find({
      'contadorExp.vigencia': {
        $gte: tomorrow,
        $lt: moment(tomorrow).endOf('day').toDate()
      }
    });

    const html = RenovarHTMl();

    despachos.forEach(despacho => {
      sendMail(html, 'Renueva tu suscripción', despacho.correo);
    });

    return res.status(200).json({
      data: despachos,
      fecha: tomorrow
    });
  } catch (error) {
    console.error('Error al realizar la búsqueda:', error);
    res.status(500).json({ message: 'Error al realizar la búsqueda' });
  }
};

const ReinicioContadorDiarioChatGpt3 = async (req, res) => {
  try {
    const now = new Date();

    const findConfiguraciones = await ConfiguracionesModel.findOne({});

    if (!findConfiguraciones) {
      return res.status(404).json({ message: 'No se encontraron las configuraciones' });
    }

    const { limiteDiarioGPT } = findConfiguraciones;

    if (!limiteDiarioGPT) {
      return res.status(404).json({ message: 'No se encontró el límite diario de GPT' });
    }

    const result = await DespachoModel.updateMany(
      { estatus: 'Activo' },
      {
        $set: {
          'contadorChatGPT.diario.limite': limiteDiarioGPT,
          'contadorChatGPT.diario.contador': 0,
          'contadorChatGPT.diario.fechaUltimoReinicio': now
        }
      }
    );

    return res.status(200).json({
      data: result,
      fecha: now
    });
  } catch (error) {
    console.error('Error al actualizar los despachos:', error);
    res.status(500).json({ message: 'Error al actualizar los despachos' });
  }
};

const DespachoSuscripcionReinicio = async (req, res) => {
  try {
    const today = moment().startOf('day').toDate();

    const despachos = await DespachoModel.find({
      'contadorExp.vigencia': {
        $lt: today
      },
      'contadorExp.paquete': { $ne: null }
    });

    despachos.forEach(despacho => {
      despacho.contadorExp.paquete = null;
      despacho.contadorExp.limite = 0;
      despacho.save();
    });

    return res.status(200).json({
      data: despachos,
      fecha: today
    });
  } catch (error) {
    console.error('Error al realizar la búsqueda:', error);
    res.status(500).json({ message: 'Error al realizar la búsqueda' });
  }
};

const createCorteDistribuidor = async (req, res) => {
  try {
    const findDistribuidores = await UsuarioModel.find({ estatus: 'Activo', tipo: 'distribuidor' }).select('_id');

    const resultados = await Promise.all(
      findDistribuidores.map(async findDistribuidor => {
        const distribuidor = findDistribuidor._id;
        const ventas = await getVentasXFechas({ distribuidor });

        if (ventas === null) {
          return null;
        }

        const { ventasTotales, totalImporte, findVentas, importeSinComision } = ventas;

        const now = moment().tz('America/Mexico_City');

        const ventaIds = findVentas.map(venta => venta._id);

        // Fecha de inicio: mes anterior, día 26 a las 00:00 horas
        const inicio = moment().tz('America/Mexico_City').subtract(1, 'month').date(26).startOf('day');

        // Fecha de fin: día 25 del mes actual a las 23:59 horas
        const fin = moment().tz('America/Mexico_City').date(25).endOf('day');

        const objCorte = {
          distribuidor,
          fecha: {
            inicio,
            fin,
            corte: now
          },
          totalVentas: ventasTotales,
          importePago: totalImporte,
          importeSinComision,
          ventas: ventaIds,
          comprobante: {
            factura: null,
            xml: null,
            pago: null,
            fechaPago: null
          },
          estatus: 'Pendiente de revisión'
        };

        const corte = new CorteModel(objCorte);

        const newCorte = await corte.save();

        await VentaModel.updateMany({ _id: { $in: ventaIds } }, { 'distribuidor.corte': newCorte._id });

        return newCorte;
      })
    );

    return res.status(201).json({ corte: resultados, message: 'Corte creado exitosamente' });
  } catch (error) {
    console.log('error:', error); // Depuración
    res.status(400).json({ message: error.message });
  }
};

module.exports = {
  AvisosAgenda,
  AvisoSuscripcion,
  ReinicioContadorDiarioChatGpt3,
  DespachoSuscripcionReinicio,
  createCorteDistribuidor
};
