const ExpedientesAgenda = require('../models/ExpedientesAgenda.js');
const UsuariosAgenda = require('../models/ExpedientesAgendaUsuarios.js');
const Expedientes = require('../models/Expedientes.js');
const moment = require('moment-timezone');
const { notificarExpedienteUsuario } = require('./Expedientes.Controller.js');

const getEvents = async (req, res) => {
  const { despacho, usuario } = req.params;
  const { expediente = null } = req.query;

  try {
    if (!usuario) { return res.status(400).json({ message: 'Usuario no especificado' }); }

    if (!despacho) { return res.status(400).json({ message: 'Despacho no especificado' }); }

    const query = {
      despacho,
      usuario
    };

    if (expediente) {
      query.expediente = expediente;
    }

    const events = await UsuariosAgenda.find(query).populate('agenda').exec();
    // ordenar por fecha ASC
    const orderedEvents = events.sort((a, b) => {
      const fechaHoraInicioA = moment(a.agenda.fecha);
      const fechaHoraInicioB = moment(b.agenda.fecha);
      const horaInicioA = a.agenda.horaInicio;
      const horaInicioB = b.agenda.horaInicio;

      if (fechaHoraInicioA.isBefore(fechaHoraInicioB)) {
        return -1;
      } else if (fechaHoraInicioA.isAfter(fechaHoraInicioB)) {
        return 1;
      } else {
        if (horaInicioA < horaInicioB) {
          return -1;
        } else if (horaInicioA > horaInicioB) {
          return 1;
        } else {
          return 0;
        }
      }
    });

    const formattedEvents = formattedEventsFun(orderedEvents);

    return res.status(200).json({ eventos: formattedEvents });
  } catch (error) {
    res.status(500).json({ message: error.message });
  }
};

const createEvent = async (req, res) => {
  const { despacho, usuario } = req.params;
  const { expediente = null, titulo, descripcion = '', color, fechaInicio, horaInicio = null, fechaFinal = null, horaFinal = null, usuarios = [], recordar = null } = req.body;

  if (!usuario) { return res.status(400).json({ message: 'Usuario no especificado' }); }

  if (!despacho) { return res.status(400).json({ message: 'Despacho no especificado' }); }

  if (!titulo) { return res.status(400).json({ message: 'Título no especificado' }); }

  if (!fechaInicio) { return res.status(400).json({ message: 'Fecha de inicio no especificada' }); }

  if (!horaInicio) {
    return res.status(400).json({ message: 'Hora de inicio no especificada' });
  }

  if (recordar && !horaInicio) {
    return res.status(400).json({ message: 'Hora de inicio no especificada' });
  }

  try {
    const fecha = moment.utc(fechaInicio).toDate();

    if (horaInicio) {
      fecha.setHours(moment(horaInicio).hours());
      fecha.setMinutes(moment(horaInicio).minutes());
    }

    const fechaFin = fechaFinal ? moment.utc(fechaFinal).toDate() : null;

    if (horaFinal && fechaFin) {
      fechaFin.setHours(moment(horaFinal).hours());
      fechaFin.setMinutes(moment(horaFinal).minutes());
    }

    const obj = {
      despacho,
      title: titulo,
      descripcion,
      fecha,
      fechaFin,
      creadoPor: usuario || null,
      fechaCreacion: moment().toDate()
    };

    if (expediente) {
      obj.expediente = expediente;
    }

    const event = await ExpedientesAgenda.create(obj);

    let fechaRecordatorioMx = null;
    if (recordar !== null) {
      fechaRecordatorioMx = moment(fecha).subtract(parseInt(recordar), 'minutes').toDate();
    }

    await UsuariosAgenda.create({
      despacho,
      expediente,
      usuario,
      fechaRecordatorio: fechaRecordatorioMx,
      agenda: event._id,
      recordar,
      color
    });

    if (usuarios.length > 0) {
      for (const _id of usuarios) {
        await UsuariosAgenda.create({
          despacho,
          expediente,
          usuario: _id,
          fechaRecordatorio: fechaRecordatorioMx,
          agenda: event._id,
          recordar,
          color
        });
      }
    }

    const tituloMovimiento = 'Nuevo evento';
    const descripcionMovimiento = `Se ha creado un nuevo evento con el título: ${titulo}. Fecha: ${moment(fecha).format('DD/MM/YYYY')}. Hora: ${moment(fecha).format('hh:mm A')}.`;

    if (expediente) { notificarExpedienteUsuario({ despacho, expediente, descripcion: descripcionMovimiento, accionRealizada: tituloMovimiento, usuario }); }

    return res.status(201).json({ evento: event, message: 'Evento creado' });
  } catch (error) {
    res.status(500).json({ message: error.message });
  }
};

const updateEvent = async (req, res) => {
  const {
    idEvento,
    idRecordatorio,
    titulo,
    estatus,
    comentario,
    descripcion,
    color,
    fechaInicio,
    horaInicio,
    fechaFinal,
    horaFinal,
    recordar,
    usuario
  } = req.body;

  try {
    const findEvento = await ExpedientesAgenda.findById(idEvento);

    if (!findEvento) { return res.status(404).json({ message: 'Evento no encontrado' }); }

    const fecha = moment.utc(fechaInicio).toDate();

    if (horaInicio) {
      fecha.setHours(moment(horaInicio).hours());
      fecha.setMinutes(moment(horaInicio).minutes());
    }

    const fechaFin = fechaFinal ? moment.utc(fechaFinal).toDate() : null;

    if (horaFinal && fechaFin) {
      fechaFin.setHours(moment(horaFinal).hours());
      fechaFin.setMinutes(moment(horaFinal).minutes());
    }

    const obj = {
      title: titulo,
      descripcion,
      fecha,
      fechaFin
    };

    const updatedEvent = await ExpedientesAgenda.findByIdAndUpdate(idEvento, obj, { new: true });

    const fechaRecordatorioMx = recordar !== null ? moment(fecha).subtract(parseInt(recordar), 'minutes').toDate() : null;

    const objRecordatorio = {
      estatus,
      comentario,
      color,
      fechaRecordatorio: fechaRecordatorioMx,
      recordar
    };

    await UsuariosAgenda.findByIdAndUpdate(idRecordatorio, objRecordatorio);

    const tituloMovimiento = 'Evento actualizado';
    const descripcionMovimiento = `Se ha actualizado el evento con el título: ${titulo}. Fecha: ${moment(fecha).format('DD/MM/YYYY')}. Hora: ${moment(fecha).format('hh:mm A')}.`;

    notificarExpedienteUsuario({ despacho: updatedEvent.despacho, expediente: updatedEvent.expediente, descripcion: descripcionMovimiento, accionRealizada: tituloMovimiento, usuario });

    return res.status(200).json({ evento: updatedEvent, message: 'Evento actualizado' });
  } catch (error) {
    res.status(500).json({ message: error.message });
  }
};

const getEventosByExpediente = async (req, res) => {
  const { expediente } = req.params;

  try {
    if (!expediente) { return res.status(400).json({ message: 'Expediente no especificado' }); }

    const query = { expediente };

    const events = await UsuariosAgenda.find(query).populate('agenda').exec();
    // ordenar por fecha ASC
    const orderedEvents = events.sort((a, b) => {
      const fechaHoraInicioA = moment(a.agenda.fecha);
      const fechaHoraInicioB = moment(b.agenda.fecha);
      const horaInicioA = a.agenda.horaInicio;
      const horaInicioB = b.agenda.horaInicio;

      if (fechaHoraInicioA.isBefore(fechaHoraInicioB)) {
        return -1;
      } else if (fechaHoraInicioA.isAfter(fechaHoraInicioB)) {
        return 1;
      } else {
        if (horaInicioA < horaInicioB) {
          return -1;
        } else if (horaInicioA > horaInicioB) {
          return 1;
        } else {
          return 0;
        }
      }
    });

    const formattedEvents = formattedEventsFun(orderedEvents);

    return res.status(200).json({ eventos: formattedEvents });
  } catch (error) {
    res.status(500).json({ message: error.message });
  }
};

const formattedEventsFun = (events) => events.map((event) => {
  const { agenda, estatus, fechaRecordatorio, comentario } = event;

  let horaInicio = null;

  if (agenda.fecha) {
    horaInicio = moment(agenda.fecha).format('hh:mm A');
    // horaInicio.setHours(moment(agenda.horaInicio).hours());
  }

  let horaFin = null;

  if (agenda.fechaFin) {
    horaFin = moment(agenda.fechaFin).format('hh:mm A');
    // horaFin.setHours(moment(agenda.horaFin).hours());
  }

  return {
    id: agenda._id,
    _id: event._id,
    title: agenda.title,
    descripcion: agenda.descripcion,
    fecha: agenda.fecha,
    horaInicio: agenda.horaInicio || horaInicio,
    horaFin: agenda.horaFin || horaFin,
    creadoPor: agenda.creadoPor,
    fechaCreacion: agenda.fechaCreacion,
    estatus,
    color: event.color,
    fechaRecordatorio,
    comentario,
    recordar: event.recordar,
    fechaFin: agenda.fechaFin

  };
});

const getEventosClientes = async (req, res) => {
  const { cliente } = req.params;

  try {
    const findExpediente = await Expedientes.find({ cliente }).select('_id');

    const expedientes = findExpediente.map((expediente) => expediente._id);

    const query = { expediente: { $in: expedientes } };

    const events = await UsuariosAgenda.find(query).populate('agenda').exec();

    const formattedEvents = formattedEventsFun(events);

    return res.status(200).json({ eventos: formattedEvents });
  } catch (error) {
    res.status(500).json({ message: error.message });
  }
};

const deleteEvento = async (req, res) => {
  const { idEvento, idRecordatorio } = req.params;

  try {
    const findEventoUsuario = await UsuariosAgenda.findById(idRecordatorio);

    if (!findEventoUsuario) { return res.status(404).json({ message: 'Evento no encontrado' }); }

    // si solo hay 1 usuario en el evento, se elimina el evento completo
    const findEventos = await UsuariosAgenda.find({ agenda: idEvento });

    if (findEventos.length === 1) {
      await ExpedientesAgenda.findByIdAndDelete(idEvento);
    }

    await UsuariosAgenda.findByIdAndDelete(idRecordatorio);

    if (findEventoUsuario.despacho && findEventoUsuario.expediente) {
      const tituloMovimiento = 'Evento eliminado';
      const descripcionMovimiento = `Se ha eliminado el evento con el título: ${findEventoUsuario.agenda.title}. Fecha: ${moment(findEventoUsuario.agenda.fecha).format('DD/MM/YYYY')}. Hora: ${moment(findEventoUsuario.agenda.fecha).format('hh:mm A')}.`;
      notificarExpedienteUsuario({ despacho: findEventoUsuario.despacho, expediente: findEventoUsuario.expediente, descripcion: descripcionMovimiento, accionRealizada: tituloMovimiento, usuario: findEventoUsuario.usuario });
    }

    return res.status(200).json({ message: 'Evento eliminado' });
  } catch (error) {
    res.status(500).json({ message: error.message });
  }
};

module.exports = {
  getEvents,
  createEvent,
  updateEvent,
  getEventosByExpediente,
  getEventosClientes,
  deleteEvento
};
