const { uploadFile, deleteFileS3, buildFileUri } = require('../config/s3.js');
const LegislacionesModel = require('../models/LegislacionesReglamentos.js');

const getLegislaciones = async (req, res) => {
  const { page = 1, search, estado, estatus } = req.query;
  const options = {
    page,
    limit: 20,
    sort: { estado: 1, nombre: 1 },
    populate: 'estado'
  };

  const query = {};

  if (estatus) {
    query.estatus = estatus;
  }

  if (search) {
    query.$or = [{ nombre: { $regex: search, $options: 'i' } }];
  }

  if (estado) {
    query.estado = estado;
  }

  if (estatus) {
    query.estatus = estatus;
  }

  try {
    const findLegislaciones = await LegislacionesModel.paginate(query, options);

    findLegislaciones.docs.map((e) => {
      if (e.adjunto) {
        e.adjunto = buildFileUri(`legislaciones-reglamentos/${e.adjunto}`);
      }
    });
    res.status(200).json({ legislaciones: findLegislaciones });
  } catch (error) {
    res.status(404).json({ message: error.message });
  }
};

const createLegislacion = async (req, res) => {
  const { nombre, estado, enlace } = req.body;
  const adjunto = req.files?.adjunto || null;

  if (!nombre) {
    return res.status(400).json({ message: 'El nombre es requerido' });
  }
  if (!estado) {
    return res.status(400).json({ message: 'El estado es requerido' });
  }
  if (!enlace) {
    return res.status(400).json({ message: 'El enlace es requerido' });
  }

  try {
    let fileName = null;

    if (adjunto) {
      const uploadAdjunto = await uploadFile(
        adjunto,
        'legislaciones-reglamentos'
      );
      fileName = uploadAdjunto.fileName;
    }

    const newLegislacion = new LegislacionesModel({
      nombre,
      estado,
      enlace,
      adjunto: fileName
    });

    await newLegislacion.save();

    res
      .status(201)
      .json({ message: 'Legislación creada', legislacion: newLegislacion });
  } catch (error) {
    res.status(409).json({ message: error.message });
  }
};

const updateLegislacion = async (req, res) => {
  const { id } = req.params;
  const { nombre, estado, enlace, estatus } = req.body;
  const adjunto = req.files?.adjunto || null;

  if (!nombre) {
    return res.status(400).json({ message: 'El nombre es requerido' });
  }

  if (!estado) {
    return res.status(400).json({ message: 'El estado es requerido' });
  }

  if (!enlace) {
    return res.status(400).json({ message: 'El enlace es requerido' });
  }

  try {
    const legislacion = await LegislacionesModel.findById(id);
    if (!legislacion) {
      return res.status(404).json({ message: 'Legislación no encontrada' });
    }

    if (adjunto) {
      const uploadAdjunto = await uploadFile(
        adjunto,
        'legislaciones-reglamentos'
      );

      if (legislacion.adjunto) {
        await deleteFileS3(`legislaciones-reglamentos/${legislacion.adjunto}`);
      }
      legislacion.adjunto = uploadAdjunto.fileName;
    }

    legislacion.nombre = nombre;
    legislacion.estado = estado;
    legislacion.enlace = enlace;
    legislacion.estatus = estatus || 'Activo';

    const updatedLegislacion = await legislacion.save();

    res.status(200).json({
      message: 'Legislación actualizada',
      legislacion: updatedLegislacion
    });
  } catch (error) {
    res.status(409).json({ message: error.message });
  }
};

const getLegislacionById = async (req, res) => {
  const { id } = req.params;
  try {
    const legislacion = await LegislacionesModel.findById(id);

    legislacion.adjunto = buildFileUri(
      `legislaciones-reglamentos/${legislacion.adjunto}`
    );
    return res.status(200).json(legislacion);
  } catch (error) {
    return res.status(404).json({ message: error.message });
  }
};

const deleteLegislacion = async (req, res) => {
  const { id } = req.params;

  try {
    await LegislacionesModel.findByIdAndDelete(id);
    res.status(200).json({ message: 'Legislación eliminada' });
  } catch (error) {
    res.status(404).json({ message: error.message });
  }
};

const generateExcel = async (req, res) => {
  const XLSX = require('xlsx');

  try {
    const findLegislaciones = await LegislacionesModel.find().populate('estado').sort({ estado: 1, nombre: 1 }).limit(10).lean();

    

    return res.status(200).json(findLegislaciones);
  } catch (error) {
    return res.status(404).json({ message: error.message });
  }
};

module.exports = {
  getLegislaciones,
  createLegislacion,
  updateLegislacion,
  getLegislacionById,
  deleteLegislacion,
  generateExcel
};
